/* copyright (C) cameo nettech {{{2
 * Filename:	proc_trace.c
 *
 * Author:	arrow <jianzhang@nettech-global.com>
 * Created at:  Mon Jun 23 08:21:21 2009
 * }}}*/
/*header files	{{{1*/
#include <linux/sysctl.h>
#include <linux/errno.h>
#include <linux/module.h>
#include <cameo/define.h>
#include <cameo/env_dump.h>
#include <cameo/proc_trace.h>
#include <cameo/tty_mirror.h>
#include <cameo/help_hotplug.h>
#include <cameo/rtl_drvver.h>
#include <cameo/led_rtl8672.h>
/*}}}*/

/*declaration		{{{1*/
static struct ctl_table_header *cameo_sysctl_header;
int dump_tty = 0;
int addnapt_enable = 0;
int flag_igmp_debug = 0;

int enable_coc = 1;

/*}}}*/

/*functions		{{{1*/
/* TODO: create proc at parent dir, build-in this proc_trace
 * under it */
static ctl_table trace_table[] = {
#ifdef CONFIG_CAMEO_TRACE_ENV_DUMP
	{
		.ctl_name	= CTL_ENV_DUMP,
		.procname	= "env_dump",
		.data		= &do_env_dump,
		.maxlen		= sizeof(int),
		.mode		= 0644,
		.proc_handler	= &proc_dointvec,
	},
#endif
#ifdef CONFIG_CAMEO_COLOR_DUMP
	{
		.ctl_name	= CTL_USE_COLOR,
		.procname	= "use_color",
		.data		= &do_use_color,
		.maxlen		= sizeof(int),
		.mode		= 0644,
		.proc_handler	= &proc_dointvec,
	},
#endif
#ifdef CONFIG_CAMEO_MEMMAP_PID
	{
		.ctl_name	= CTL_MEMMAP_PID_PHY,
		.procname	= "memmap_pid_phy",
		.data		= &phy_memmap_pid,
		.maxlen		= sizeof(int),
		.mode		= 0644,
		.proc_handler	= &proc_dointvec,
	},
	{
		.ctl_name	= CTL_MEMMAP_PID_STA,
		.procname	= "memmap_pid_sta",
		.data		= &sta_memmap_pid,
		.maxlen		= sizeof(int),
		.mode		= 0644,
		.proc_handler	= &proc_dointvec,
	},
	{
		.ctl_name	= CTL_MEMMAP_PID_CNT,
		.procname	= "memmap_pid_cnt",
		.data		= &cnt_memmap_pid,
		.maxlen		= sizeof(int),
		.mode		= 0644,
		.proc_handler	= &proc_dointvec,
	},
	{
		.ctl_name	= CTL_MEMMAP_DUMP_STACK,
		.procname	= "memmap_stack",
		.data		= &do_stack_dump,
		.maxlen		= sizeof(int),
		.mode		= 0644,
		.proc_handler	= &proc_dointvec,
	},
#endif
#ifdef CONFIG_CAMEO_TTY_MIRROR
	{
		.ctl_name	= CTL_TTY_MIRROR_PRINTK,
		.procname	= "tty_mirror_printk",
		.data		= &tty_mirror_printk,
		.maxlen		= sizeof(int),
		.mode		= 0644,
		.proc_handler	= &proc_dointvec,
	},
	{
		.ctl_name	= CTL_TTY_MIRROR_MIRROR,
		.procname	= "tty_mirror_dev",
		.data		= &tty_mirror_dev,
		.maxlen		= MAX_MIRROR_TTY_NAME_SIZE,
		.mode		= 0644,
		.proc_handler	= &proc_dostring,
	},
#endif
#ifdef CONFIG_CAMEO_HOTPLUG
	{
		.ctl_name	= CTL_HELP_HOTPLUG_BIN,
		.procname	= "help_hotplug",
		.data		= &bin_help_hotplug,
		.maxlen		= MAX_HELP_HOTPLUG_SIZE,
		.mode		= 0644,
		.proc_handler	= &proc_dostring,
	},
#endif
#ifdef CONFIG_LOG_NETFILTER
	{
		.ctl_name	= CTL_LOG_NETFILTER,
		.procname	= "log_netfilter",
		.data		= &log_netfilter_enable,
		.maxlen		= sizeof(int),
		.mode		= 0644,
		.proc_handler	= &proc_dointvec,
	},
#endif
	/* use for v2, so macro is not required */
	{
		.ctl_name	= CTL_ADDNAPT_ENABLE,
		.procname	= "addnapt_enable",
		.data		= &addnapt_enable,
		.maxlen		= sizeof(int),
		.mode		= 0644,
		.proc_handler	= &proc_dointvec,
	},
#ifdef CONFIG_RTL_DRVVER
	{
		.ctl_name	= CTL_DRVVER_IPACCEL,
		.procname	= "drvver_ipaccel",
		.data		= &rtl_ipaccel_drvver,
		.maxlen		= sizeof(rtl_ipaccel_drvver),
		.mode		= 0444,
		.proc_handler	= &proc_dostring,
	},
	{
		.ctl_name	= CTL_DRVVER_ETHER,
		.procname	= "drvver_ether",
		.data		= &rtl_ether_drvver,
		.maxlen		= sizeof(rtl_ether_drvver),
		.mode		= 0444,
		.proc_handler	= &proc_dostring,
	},
	{
		.ctl_name	= CTL_DRVVER_SAR,
		.procname	= "drvver_sar",
		.data		= &rtl_sar_drvver,
		.maxlen		= sizeof(rtl_sar_drvver),
		.mode		= 0444,
		.proc_handler	= &proc_dostring,
	},
#endif
	{
		.ctl_name	= CTL_DRVVER_SAR,
		.procname	= "igmp_debug",
		.data		= &flag_igmp_debug,
		.maxlen		= sizeof(flag_igmp_debug),
		.mode		= 0666,
		.proc_handler	= &proc_dointvec,
	},
	{
		.ctl_name	= CTL_RTL_IPQOSFLAG,
		.procname	= "rtl_ipqos",
		.data		= &enable_ipqos,
		.maxlen		= sizeof(enable_ipqos),
		.mode		= 0666,
		.proc_handler	= &proc_dointvec,
	},
#ifdef CONFIG_RTL8672_LED_HIJACK
	{
		.ctl_name	= CTL_RTL_LED_HIJACK,
		.procname	= "led_hijack",
		.data		= &enable_led_hijack,
		.maxlen		= sizeof(enable_led_hijack),
		.mode		= 0666,
		.proc_handler	= &proc_dointvec,
	},
#endif
#ifdef CONFIG_RTL8192CD_SOFTLED
	{
		.ctl_name	= CTL_RTL_LED_POLL_NUM,
		.procname	= "led_poll_scalar",
		.data		= &led_poll_num,
		.maxlen		= sizeof(led_poll_num),
		.mode		= 0666,
		.proc_handler	= &proc_dointvec,
	},
	{
		.ctl_name	= CTL_RTL_LED_POLL_INTV,
		.procname	= "led_poll_interval",
		.data		= &poll_interval,
		.maxlen		= sizeof(poll_interval),
		.mode		= 0666,
		.proc_handler	= &proc_dointvec,
	},
	{
		.ctl_name	= CTL_RTL_LED_POLL_DELAY,
		.procname	= "led_poll_delay",
		.data		= &led_poll_delay,
		.maxlen		= sizeof(led_poll_delay),
		.mode		= 0666,
		.proc_handler	= &proc_dointvec,
	},
#endif
	{
		.ctl_name	= CTL_DRVVER_SAR,
		.procname	= "enable_coc",
		.data		= &enable_coc,
		.maxlen		= sizeof(enable_coc),
		.mode		= 0666,
		.proc_handler	= &proc_dointvec,
	},

	{ .ctl_name = 0 }
};

static ctl_table cameo_sysctl_table[] = {
	{
		.ctl_name	= 0,
		.procname	= "cameo",
		.mode		= 0555,
		.child		= trace_table,
	},
	{ .ctl_name = 0 }
};

static int __init trace_cameo_init(void)
{
	cameo_sysctl_header = register_sysctl_table(cameo_sysctl_table, 0);
	if (cameo_sysctl_header == NULL) {
		seepd("trace sysctl reg fail\n");
		return -EFAULT;
	}
	return 0;
}

void __exit trace_cameo_fini(void)
{
	unregister_sysctl_table(cameo_sysctl_header);
}

module_init(trace_cameo_init);
module_exit(trace_cameo_fini);
/* vim:fdm=marker:ts=8:ft=c:norl:fdl=1:
 * }}}*/

